package gs.smolban.model.users

import cats.Show
import gs.smolban.model.CreatedAt
import gs.uuid.v0.UUID

/** Represents a user that can interact with Smolban.
  *
  * @param id
  *   Unique identifier of this user.
  * @param createdAt
  *   The instant at which this user was created.
  * @param username
  *   The (unique) [[Username]] of this user.
  * @param designation
  *   The [[User.Designation]] of this user.
  * @param roles
  *   List of [[Role]] assigned to this user.
  * @param status
  *   The current [[User.Status]] of this user.
  */
case class User(
  id: User.Id,
  createdAt: CreatedAt,
  username: Username,
  designation: User.Designation,
  roles: List[Role],
  status: User.Status
)

object User:

  given CanEqual[User, User] = CanEqual.derived

  /** Unique identifier for a [[User]]. This is an opaque type for a UUID.
    */
  opaque type Id = UUID

  object Id:

    /** Instantiate a new [[User.Id]].
      *
      * @param id
      *   The underlying UUID.
      * @return
      *   The new [[User.Id]] instance.
      */
    def apply(id: UUID): Id = id

    given CanEqual[Id, Id] = CanEqual.derived

    given Show[Id] = _.toUUID().withoutDashes()

    extension (id: Id)
      /** Unwrap this User ID.
        *
        * @return
        *   The underlying UUID value.
        */
      def toUUID(): UUID = id

  end Id

  /** Enumeration that describes the designation of a [[User]].
    */
  sealed abstract class Designation(val name: String)

  object Designation:

    /** Regular users are typically human, and are expected to be using the Web
      * UI and _possibly_ APIs.
      */
    case object Regular extends Designation("regular")

    /** Service users are intended for use by API-consuming services. They are
      * not suitable for human/UI interactive use.
      */
    case object Service extends Designation("service")

  end Designation

  /** Enumeration that describes the status of a [[User]] in Smolban.
    */
  sealed abstract class Status(val name: String)

  object Status:

    given CanEqual[Status, Status] = CanEqual.derived

    given Show[Status] = _.name

    case object Active    extends Status("active")
    case object Suspended extends Status("suspended")
    case object Off       extends Status("off")

end User
