package gs.smolban.model.users

import cats.Show

/** Opaque type for String that represents a unique username in Smolban.
  */
opaque type Username = String

object Username:

  /** In Smolban, a [[Username]] must be at most 32 characters long.
    */
  val MaximumLength: Int = 32

  /** In Smolban, a [[Username]] must be at least 3 characters long.
    */
  val MinimumLength: Int = 3

  given CanEqual[Username, Username] = CanEqual.derived

  /** Validate some candidate string, producing a [[Username]] if valid. Smolban
    * usernames must be:
    *
    *   - At least 3 characters long.
    *   - At most 32 characters long.
    *   - Non-blank -- non-whitespace characters must be used.
    *
    * @param candidate
    *   The candidate string to evaluate.
    * @return
    *   The [[Username]], or `None` if the candidate was invalid.
    */
  def validate(candidate: String): Option[Username] =
    if isValid(candidate) then Some(candidate) else None

  private def isValid(candidate: String): Boolean =
    isValidSize(candidate) && isNonBlank(candidate)

  private def isValidSize(candidate: String): Boolean =
    candidate.length() >= MinimumLength && candidate.length() <= MaximumLength

  private def isNonBlank(candidate: String): Boolean =
    !candidate.isBlank()

  given Show[Username] = u => u

end Username
